@@ -16,26 +16,22 @@
 """
 
 
-from xml.etree import ElementTree as ET
 from copy import copy as copy_obj
+from functools import singledispatch
+from typing import Any, Tuple, Union
+from urllib.parse import urlparse
+from xml.etree import ElementTree as ET
 
-try:
-    from functools import singledispatch
-except ImportError:
-    from singledispatch import singledispatch
-
-from ...compat import is_py2, urlparse
-from ...exceptions import PluginError
+from streamlink.exceptions import PluginError
 
 __all__ = [
     "any", "all", "filter", "get", "getattr", "hasattr", "length", "optional",
-    "transform", "text", "union", "url", "startswith", "endswith", "contains",
+    "transform", "text", "union", "union_get", "url", "startswith", "endswith", "contains",
     "xml_element", "xml_find", "xml_findall", "xml_findtext",
     "validate", "Schema", "SchemaContainer"
 ]
 
-#: Alias for text type on each Python version
-text = is_py2 and basestring or str
+text = str
 
 # References to original functions that we override in this module
 _all = all
@@ -55,34 +51,28 @@ def _is_re_match(value):
 class any(tuple):
     """At least one of the schemas must be valid."""
     def __new__(cls, *args):
-        return super(any, cls).__new__(cls, args)
+        return super().__new__(cls, args)
 
 
 class all(tuple):
     """All schemas must be valid."""
     def __new__(cls, *args):
-        return super(all, cls).__new__(cls, args)
+        return super().__new__(cls, args)
 
 
-class SchemaContainer(object):
+class SchemaContainer:
     def __init__(self, schema):
         self.schema = schema
 
 
-class transform(object):
+class transform:
     """Applies function to value to transform it."""
 
     def __init__(self, func):
-        # text is an alias for basestring on Python 2, which cannot be
-        # instantiated and therefore can't be used to transform the value,
-        # so we force to unicode instead.
-        if is_py2 and func == text:
-            func = unicode
-
         self.func = func
 
 
-class optional(object):
+class optional:
     """An optional key used in a dict or union-dict."""
 
     def __init__(self, key):
@@ -97,7 +87,13 @@ class attr(SchemaContainer):
     """Validates an object's attributes."""
 
 
-class xml_element(object):
+class union_get:
+    def __init__(self, *keys, seq=tuple):
+        self.keys = keys
+        self.seq = seq
+
+
+class xml_element:
     """A XML element."""
 
     def __init__(self, tag=None, text=None, attrib=None):
@@ -144,32 +140,43 @@ def contains(string):
     """Checks if the string value contains another string."""
     def contains_str(value):
         validate(text, value)
-        if not string in value:
+        if string not in value:
             raise ValueError("'{0}' does not contain '{1}'".format(value, string))
         return True
 
     return contains_str
 
 
-def get(item, default=None):
+def get(item: Union[Any, Tuple[Any]], default: Any = None, strict: bool = False):
     """Get item from value (value[item]).
 
-    If the item is not found, return the default.
+    Unless strict is set to True, item can be a tuple of items for recursive lookups.
+
+    If the item is not found in the last object of a recursive lookup, return the default.
 
     Handles XML elements, regex matches and anything that has __getitem__.
     """
 
-    def getter(value):
-        if ET.iselement(value):
-            value = value.attrib
+    if type(item) is not tuple or strict:
+        item = (item,)
 
+    def getter(value):
+        idx = 0
         try:
-            # Use .group() if this is a regex match object
-            if _is_re_match(value):
-                return value.group(item)
-            else:
-                return value[item]
+            for key in item:
+                if ET.iselement(value):
+                    value = value.attrib
+                # Use .group() if this is a regex match object
+                elif _is_re_match(value):
+                    value = value.group(key)
+                else:
+                    value = value[key]
+                idx += 1
+            return value
         except (KeyError, IndexError):
+            # only return default value on last item in nested lookup
+            if idx < len(item) - 1:
+                raise ValueError(f"Object \"{value}\" does not have item \"{key}\"")
             return default
         except (TypeError, AttributeError) as err:
             raise ValueError(err)
@@ -222,12 +229,6 @@ def map(func):
     Supports both dicts and sequences, key/value pairs are
     expanded when applied to a dict.
     """
-    # text is an alias for basestring on Python 2, which cannot be
-    # instantiated and therefore can't be used to transform the value,
-    # so we force to unicode instead.
-    if is_py2 and text == func:
-        func = unicode
-
     def expand_kv(kv):
         return func(*kv)
 
@@ -434,6 +435,11 @@ def validate_attr(schema, value):
     return new
 
 
+@validate.register(union_get)
+def validate_union_from(schema, value):
+    return schema.seq(validate(get(k), value) for k in schema.keys)
+
+
 @singledispatch
 def validate_union(schema, value):
     raise ValueError("Invalid union type: {0}".format(type(schema).__name__))
@@ -471,7 +477,7 @@ def validate_unions(schema, value):
     return validate_union(schema.schema, value)
 
 
-class Schema(object):
+class Schema:
     """Wraps a validator schema into a object."""
 
     def __init__(self, *schemas):
