@@ -1,23 +1,18 @@
-from __future__ import print_function
-
 import logging
 import random
 import re
 
-from streamlink.compat import urljoin
-from streamlink.plugin import Plugin, PluginArguments, PluginArgument
-from streamlink.plugin.api import http
-from streamlink.plugin.api import useragents
-from streamlink.plugin.api import validate
+from streamlink.plugin import Plugin, PluginArgument, PluginArguments
+from streamlink.plugin.api import useragents, validate
 from streamlink.plugin.api.utils import itertags
-from streamlink.stream import HLSStream
-from streamlink.stream import HTTPStream
+from streamlink.stream import HLSStream, HTTPStream
 from streamlink.stream.ffmpegmux import MuxedStream
+from streamlink.utils.l10n import Localization
 
 log = logging.getLogger(__name__)
 
 
-class Experience(object):
+class Experience:
     CSRF_NAME = "csrfmiddlewaretoken"
     login_url = "https://www.funimation.com/log-in/"
     api_base = "https://www.funimation.com/api"
@@ -34,10 +29,12 @@ class Experience(object):
          "user": {"id": int}}
     ))
 
-    def __init__(self, experience_id):
+    def __init__(self, session, experience_id):
         """
+        :param session: streamlink session
         :param experience_id: starting experience_id, may be changed later
         """
+        self.session = session
         self.experience_id = experience_id
         self._language = None
         self.cache = {}
@@ -47,11 +44,19 @@ class Experience(object):
         headers = kwargs.pop("headers", {})
         if self.token:
             headers.update({"Authorization": "Token {0}".format(self.token)})
-            http.cookies.update({"src_token": self.token})
+            self.session.http.cookies.update({"src_token": self.token})
 
         log.debug("Making {0}request to {1}".format("authorized " if self.token else "", url))
 
-        return http.request(method, url, *args, headers=headers, **kwargs)
+        res = self.session.http.request(method, url, *args, headers=headers, **kwargs)
+        if "_Incapsula_Resource" in res.text:
+            log.error(
+                "This page is protected by Incapsula, please see "
+                "https://github.com/streamlink/streamlink/issues/2088"
+                " for a workaround."
+            )
+            return
+        return res
 
     def get(self, *args, **kwargs):
         return self.request("GET", *args, **kwargs)
@@ -69,8 +74,9 @@ class Experience(object):
         api_url = self.show_api_url.format(experience_id=self.experience_id)
         log.debug("Requesting experience data: {0}".format(api_url))
         res = self.get(api_url)
-        data = http.json(res)
-        self.cache[self.experience_id] = data
+        if res:
+            data = self.session.http.json(res)
+            self.cache[self.experience_id] = data
 
     @property
     def show_info(self):
@@ -84,12 +90,13 @@ class Experience(object):
         Search for the episode with the requested experience Id
         :return:
         """
-        for season in self.show_info["seasons"]:
-            for episode in season["episodes"]:
-                for lang in episode["languages"].values():
-                    for alpha in lang["alpha"].values():
-                        if alpha["experienceId"] == self.experience_id:
-                            return episode
+        if self.show_info:
+            for season in self.show_info["seasons"]:
+                for episode in season["episodes"]:
+                    for lang in episode["languages"].values():
+                        for alpha in lang["alpha"].values():
+                            if alpha["experienceId"] == self.experience_id:
+                                return episode
 
     @property
     def language(self):
@@ -126,10 +133,10 @@ class Experience(object):
         """
         api_url = self.sources_api_url.format(experience_id=self.experience_id)
         res = self.get(api_url, params={"pinst_id": self.pinst_id})
-        return http.json(res)
+        return self.session.http.json(res)
 
     def login_csrf(self):
-        r = http.get(self.login_url)
+        r = self.session.http.get(self.login_url)
         for input in itertags(r.text, "input"):
             if input.attributes.get("name") == self.CSRF_NAME:
                 return input.attributes.get("value")
@@ -140,7 +147,7 @@ class Experience(object):
                       data={'username': email, 'password': password, self.CSRF_NAME: self.login_csrf()},
                       raise_for_status=False,
                       headers={"Referer": "https://www.funimation.com/log-in/"})
-        d = http.json(r, schema=self.login_schema)
+        d = self.session.http.json(r, schema=self.login_schema)
         self.token = d.get("token", None)
         return self.token is not None
 
@@ -170,15 +177,7 @@ class FunimationNow(Plugin):
             Default is "english".
             """
         ),
-        PluginArgument(
-            "mux-subtitles",
-            argument_name="funimation-mux-subtitles",
-            action="store_true",
-            help="""
-            Enable automatically including available subtitles in to the output
-            stream.
-            """
-        )
+        PluginArgument("mux-subtitles", is_global=True)
     )
 
     url_re = re.compile(r"""
@@ -192,95 +191,86 @@ class FunimationNow(Plugin):
         return cls.url_re.match(url) is not None
 
     def _get_streams(self):
-        http.headers = {"User-Agent": useragents.CHROME}
-        res = http.get(self.url)
+        self.session.http.headers = {"User-Agent": useragents.CHROME}
+        res = self.session.http.get(self.url)
 
         # remap en to english, and ja to japanese
         rlanguage = {"en": "english", "ja": "japanese"}.get(self.get_option("language").lower(),
                                                             self.get_option("language").lower())
         if "_Incapsula_Resource" in res.text:
-            self.bypass_incapsula(res)
-            res = http.get(self.url)
+            log.error(
+                "This page is protected by Incapsula, please see "
+                "https://github.com/streamlink/streamlink/issues/2088"
+                " for a workaround."
+            )
+            return
+
+        if "Out of Territory" in res.text:
+            log.error("The content requested is not available in your territory.")
+            return
 
         id_m = self.experience_id_re.search(res.text)
         experience_id = id_m and int(id_m.group(1))
         if experience_id:
-            log.debug("Found experience ID: {0}", experience_id)
-            exp = Experience(experience_id)
+            log.debug(f"Found experience ID: {experience_id}")
+            exp = Experience(self.session, experience_id)
             if self.get_option("email") and self.get_option("password"):
                 if exp.login(self.get_option("email"), self.get_option("password")):
-                    log.info("Logged in to Funimation as {0}", self.get_option("email"))
+                    log.info(f"Logged in to Funimation as {self.get_option('email')}")
                 else:
                     log.warning("Failed to login")
 
-            log.debug("Found episode: {0}", exp.episode_info["episodeTitle"])
-            log.debug("  has languages: {0}", ", ".join(exp.episode_info["languages"].keys()))
-            log.debug("  requested language: {0}", rlanguage)
-            log.debug("  current language:   {0}", exp.language)
-            if rlanguage != exp.language:
-                log.debug("switching language to: {0}", rlanguage)
-                exp.set_language(rlanguage)
-                if exp.language != rlanguage:
-                    log.warning("Requested language {0} is not available, continuing with {1}",
-                                rlanguage, exp.language)
-                else:
-                    log.debug("New experience ID: {0}", exp.experience_id)
-
-            subtitles = None
-            stream_metadata = {}
-            disposition = {}
-            for subtitle in exp.subtitles():
-                log.debug("Subtitles: {0}", subtitle["src"])
-                if subtitle["src"].endswith(".vtt") or subtitle["src"].endswith(".srt"):
-                    sub_lang = {"en": "eng", "ja": "jpn"}[subtitle["language"]]
-                    # pick the first suitable subtitle stream
-                    subtitles = subtitles or HTTPStream(self.session, subtitle["src"])
-                    stream_metadata["s:s:0"] = ["language={0}".format(sub_lang)]
-                stream_metadata["s:a:0"] = ["language={0}".format(exp.language_code)]
-
-            sources = exp.sources()
-            if 'errors' in sources:
-                for error in sources['errors']:
-                    log.error("{0} : {1}".format(error['title'], error['detail']))
-                return
-
-            for item in sources["items"]:
-                url = item["src"]
-                if ".m3u8" in url:
-                    for q, s in HLSStream.parse_variant_playlist(self.session, url).items():
-                        if self.get_option("mux_subtitles") and subtitles:
-                            yield q, MuxedStream(self.session, s, subtitles, metadata=stream_metadata,
-                                                 disposition=disposition)
-                        else:
-                            yield q, s
-                elif ".mp4" in url:
-                    # TODO: fix quality
-                    s = HTTPStream(self.session, url)
-                    if self.get_option("mux_subtitles") and subtitles:
-                        yield self.mp4_quality, MuxedStream(self.session, s, subtitles, metadata=stream_metadata,
-                                                            disposition=disposition)
+            if exp.episode_info:
+                log.debug(f"Found episode: {exp.episode_info['episodeTitle']}")
+                log.debug(f"  has languages: {', '.join(exp.episode_info['languages'].keys())}")
+                log.debug(f"  requested language: {rlanguage}")
+                log.debug(f"  current language:   {exp.language}")
+                if rlanguage != exp.language:
+                    log.debug(f"switching language to: {rlanguage}")
+                    exp.set_language(rlanguage)
+                    if exp.language != rlanguage:
+                        log.warning(f"Requested language {rlanguage} is not available, continuing with {exp.language}")
                     else:
-                        yield self.mp4_quality, s
+                        log.debug(f"New experience ID: {exp.experience_id}")
+
+                subtitles = None
+                stream_metadata = {}
+                disposition = {}
+                for subtitle in exp.subtitles():
+                    log.debug(f"Subtitles: {subtitle['src']}")
+                    if subtitle["src"].endswith(".vtt") or subtitle["src"].endswith(".srt"):
+                        sub_lang = Localization.get_language(subtitle["language"]).alpha3
+                        # pick the first suitable subtitle stream
+                        subtitles = subtitles or HTTPStream(self.session, subtitle["src"])
+                        stream_metadata["s:s:0"] = ["language={0}".format(sub_lang)]
+                    stream_metadata["s:a:0"] = ["language={0}".format(exp.language_code)]
+
+                sources = exp.sources()
+                if 'errors' in sources:
+                    for error in sources['errors']:
+                        log.error("{0} : {1}".format(error['title'], error['detail']))
+                    return
+
+                for item in sources["items"]:
+                    url = item["src"]
+                    if ".m3u8" in url:
+                        for q, s in HLSStream.parse_variant_playlist(self.session, url).items():
+                            if self.get_option("mux_subtitles") and subtitles:
+                                yield q, MuxedStream(self.session, s, subtitles, metadata=stream_metadata,
+                                                     disposition=disposition)
+                            else:
+                                yield q, s
+                    elif ".mp4" in url:
+                        # TODO: fix quality
+                        s = HTTPStream(self.session, url)
+                        if self.get_option("mux_subtitles") and subtitles:
+                            yield self.mp4_quality, MuxedStream(self.session, s, subtitles, metadata=stream_metadata,
+                                                                disposition=disposition)
+                        else:
+                            yield self.mp4_quality, s
 
         else:
             log.error("Could not find experience ID?!")
 
-    def bypass_incapsula(self, res):
-        log.info("Attempting to by-pass Incapsula...")
-        self.clear_cookies(lambda c: "incap" in c.name)
-        for m in re.finditer(r'''"([A-Z0-9]+)"''', res.text):
-            d = m.group(1)
-            # decode the encoded blob to text
-            js = "".join(map(lambda i: chr(int(i, 16)), [d[x:x + 2] for x in range(0, len(d), 2)]))
-            jsm = re.search(r'''"GET","([^"]+)''', js)
-            url = jsm and jsm.group(1)
-            if url:
-                log.debug("Found Incapsula auth URL: {0}", url)
-                res = http.get(urljoin(self.url, url))
-                success = res.status_code == 200
-                if success:
-                    self.save_cookies(lambda c: "incap" in c.name)
-                return success
-
 
 __plugin__ = FunimationNow
