@@ -1,10 +1,7 @@
-from __future__ import unicode_literals
-
 import logging
 import re
 
-from streamlink import PluginError
-from streamlink.compat import urljoin
+from streamlink.exceptions import PluginError
 from streamlink.plugin import Plugin
 from streamlink.plugin.api import useragents
 from streamlink.stream import DASHStream, HLSStream
@@ -14,13 +11,11 @@ log = logging.getLogger(__name__)
 
 class TF1(Plugin):
     url_re = re.compile(r"https?://(?:www\.)?(?:tf1\.fr/([\w-]+)/direct|(lci).fr/direct)/?")
-    api_url_base = "https://delivery.tf1.fr/mytf1-wrd/"
-    token = "07e45841-a17a-47cf-af64-a42311bdcc3d"
+    api_url = "https://mediainfo.tf1.fr/mediainfocombo/{}?context=MYTF1&pver=4001000"
 
     def api_call(self, channel, useragent=useragents.CHROME):
-        url = urljoin(self.api_url_base, "L_" + channel.upper())
+        url = self.api_url.format("L_" + channel.upper())
         req = self.session.http.get(url,
-                                    params=dict(token=self.token),
                                     headers={"User-Agent": useragent})
         return self.session.http.json(req)
 
@@ -28,11 +23,11 @@ class TF1(Plugin):
         for useragent in [useragents.CHROME, useragents.IPHONE_6]:
             data = self.api_call(channel, useragent)
 
-            if data.get("error"):
-                log.error("Failed to get {format} stream for {0}: {error} ({code})".format(channel, **data))
-            else:
-                log.debug("Got {format} stream {url}".format(**data))
-                yield data['format'], data['url']
+            if 'delivery' not in data or 'url' not in data['delivery']:
+                continue
+
+            log.debug("Got {format} stream {url}".format(**data['delivery']))
+            yield data['delivery']['format'], data['delivery']['url']
 
     @classmethod
     def can_handle_url(cls, url):
@@ -46,11 +41,17 @@ class TF1(Plugin):
             for sformat, url in self.get_stream_urls(channel):
                 try:
                     if sformat == "dash":
-                        for s in DASHStream.parse_manifest(self.session, url, headers={"User-Agent": useragents.CHROME}).items():
-                            yield s
+                        yield from DASHStream.parse_manifest(
+                            self.session,
+                            url,
+                            headers={"User-Agent": useragents.CHROME}
+                        ).items()
                     if sformat == "hls":
-                        for s in HLSStream.parse_variant_playlist(self.session, url).items():
-                            yield s
+                        yield from HLSStream.parse_variant_playlist(
+                            self.session,
+                            url,
+                            headers={"User-Agent": useragents.IPHONE},
+                        ).items()
                 except PluginError as e:
                     log.error("Could not open {0} stream".format(sformat))
                     log.debug("Failed with error: {0}".format(e))
