@@ -1,16 +1,14 @@
-import re
 import logging
-
+import math
+import re
 from binascii import unhexlify
 from collections import namedtuple
 from datetime import timedelta
 from itertools import starmap
+from urllib.parse import urljoin, urlparse
 
 from isodate import parse_datetime
 
-from streamlink.compat import urljoin, urlparse
-
-
 log = logging.getLogger(__name__)
 
 __all__ = ["load", "M3U8Parser"]
@@ -45,7 +43,7 @@ Resolution = namedtuple("Resolution", "width height")
 Segment = namedtuple("Segment", "uri duration title key discontinuity byterange date map")
 
 
-class M3U8(object):
+class M3U8:
     def __init__(self):
         self.is_endlist = False
         self.is_master = False
@@ -80,7 +78,7 @@ class M3U8(object):
         return daterange.start_date <= date
 
 
-class M3U8Parser(object):
+class M3U8Parser:
     _extinf_re = re.compile(r"(?P<duration>\d+(\.\d+)?)(,(?P<title>.+))?")
     _attr_re = re.compile(r"([A-Z\-]+)=(\d+\.\d+|0x[0-9A-z]+|\d+x\d+|\d+|\"(.+?)\"|[0-9A-z\-]+)")
     _range_re = re.compile(r"(?P<range>\d+)(@(?P<offset>.+))?")
@@ -97,7 +95,7 @@ class M3U8Parser(object):
 
         bandwidth = streaminf.get("BANDWIDTH")
         if bandwidth:
-            bandwidth = float(bandwidth)
+            bandwidth = round(int(bandwidth), 1 - int(math.log10(int(bandwidth))))
 
         resolution = streaminf.get("RESOLUTION")
         if resolution:
