@@ -1,7 +1,6 @@
 import locale
 import logging
 
-from streamlink.compat import is_py2
 
 try:
     from iso639 import languages
@@ -20,7 +19,7 @@ DEFAULT_LANGUAGE_CODE = "{0}_{1}".format(DEFAULT_LANGUAGE, DEFAULT_COUNTRY)
 log = logging.getLogger(__name__)
 
 
-class Country(object):
+class Country:
     def __init__(self, alpha2, alpha3, numeric, name, official_name=None):
         self.alpha2 = alpha2
         self.alpha3 = alpha3
@@ -41,25 +40,23 @@ class Country(object):
             raise LookupError("Invalid country code: {0}".format(country))
 
     def __eq__(self, other):
-        return ((self.alpha2 and self.alpha2 == other.alpha2) or
-                (self.alpha3 and self.alpha3 == other.alpha3) or
-                (self.numeric and self.numeric == other.numeric))
+        return (
+            (self.alpha2 and self.alpha2 == other.alpha2)
+            or (self.alpha3 and self.alpha3 == other.alpha3)
+            or (self.numeric and self.numeric == other.numeric)
+        )
 
     def __str__(self):
-        if is_py2:
-            return self.__unicode__().encode("utf8")
-        else:
-            return self.__unicode__()
-
-    def __unicode__(self):
-        return u"Country({0!r}, {1!r}, {2!r}, {3!r}, official_name={4!r})".format(self.alpha2,
-                                                                                  self.alpha3,
-                                                                                  self.numeric,
-                                                                                  self.name,
-                                                                                  self.official_name)
+        return "Country({0!r}, {1!r}, {2!r}, {3!r}, official_name={4!r})".format(
+            self.alpha2,
+            self.alpha3,
+            self.numeric,
+            self.name,
+            self.official_name
+        )
 
 
-class Language(object):
+class Language:
     def __init__(self, alpha2, alpha3, name, bibliographic=None):
         self.alpha2 = alpha2
         self.alpha3 = alpha3
@@ -70,46 +67,55 @@ class Language(object):
     def get(cls, language):
         try:
             if PYCOUNTRY:
-                c = languages.lookup(language)
-                return Language(c.alpha_2, c.alpha_3, c.name, getattr(c, "bibliographic", None))
+                lang = (languages.get(alpha_2=language)
+                        or languages.get(alpha_3=language)
+                        or languages.get(bibliographic=language)
+                        or languages.get(name=language))
+                if not lang:
+                    raise KeyError(language)
+                return Language(
+                    # some languages don't have an alpha_2 code
+                    getattr(lang, "alpha_2", ""),
+                    lang.alpha_3,
+                    lang.name,
+                    getattr(lang, "bibliographic", "")
+                )
             else:
-                l = None
+                lang = None
                 if len(language) == 2:
-                    l = languages.get(alpha2=language)
+                    lang = languages.get(alpha2=language)
                 elif len(language) == 3:
                     for code_type in ['part2b', 'part2t', 'part3']:
                         try:
-                            l = languages.get(**{code_type: language})
+                            lang = languages.get(**{code_type: language})
                             break
                         except KeyError:
                             pass
-                    if not l:
+                    if not lang:
                         raise KeyError(language)
                 else:
                     raise KeyError(language)
-                return Language(l.alpha2, l.part3, l.name, l.part2b or l.part2t)
+                return Language(lang.alpha2, lang.part3, lang.name, lang.part2b or lang.part2t)
         except (LookupError, KeyError):
             raise LookupError("Invalid language code: {0}".format(language))
 
     def __eq__(self, other):
-        return ((self.alpha2 and self.alpha2 == other.alpha2) or
-                (self.alpha3 and self.alpha3 == other.alpha3) or
-                (self.bibliographic and self.bibliographic == other.bibliographic))
+        return (
+            (self.alpha2 and self.alpha2 == other.alpha2)
+            or (self.alpha3 and self.alpha3 == other.alpha3)
+            or (self.bibliographic and self.bibliographic == other.bibliographic)
+        )
 
     def __str__(self):
-        if is_py2:
-            return self.__unicode__().encode("utf8")
-        else:
-            return self.__unicode__()
-
-    def __unicode__(self):
-        return u"Language({0!r}, {1!r}, {2!r}, bibliographic={3!r})".format(self.alpha2,
-                                                                            self.alpha3,
-                                                                            self.name,
-                                                                            self.bibliographic)
+        return "Language({0!r}, {1!r}, {2!r}, bibliographic={3!r})".format(
+            self.alpha2,
+            self.alpha3,
+            self.name,
+            self.bibliographic
+        )
 
 
-class Localization(object):
+class Localization:
     def __init__(self, language_code=None):
         self._language_code = None
         self.country = None
@@ -137,7 +143,7 @@ class Localization(object):
                 language_code = None
             if language_code is None or language_code == "C":
                 # cannot be determined
-                language_code = DEFAULT_LANGUAGE
+                language_code = DEFAULT_LANGUAGE_CODE
 
         try:
             self.language, self.country = self._parse_locale_code(language_code)
